use crate::app::series::Source;

use leptos::*;
use leptos_router::*;

#[component]
pub fn AddSeries() -> impl IntoView {
    let add_series = expect_context::<Action<AddSeries, Result<(), ServerFnError>>>();
    view! {
        <div class="card p-3">
            <div class="card-heading">
                <A href="/"
                    attr:class="btn btn-close"
                    attr:style="float: right;"
                >
                    <span style="display: none;">x</span>
                </A>
                <h2>New Series</h2>
            </div>
            <div class="card-body">
                <ActionForm action=add_series>
                    <div class="input-group mb-3">
                        <label class="input-group-text" for="name">Name</label>
                        <input type="text"
                            name="name"
                            class="form-control"
                        />
                    </div>
                    <div class="input-group mb-3">
                        <label class="input-group-text" for="source">Source</label>
                        <select class="form-control" name="source">
                            <option value=Source::Disney.to_string()>{Source::Disney.to_string()}</option>
                            <option value=Source::Hulu.to_string()>{Source::Hulu.to_string()}</option>
                            <option value=Source::Paramount.to_string()>{Source::Paramount.to_string()}</option>
                            <option value=Source::Prime.to_string()>{Source::Prime.to_string()}</option>
                            <option value=Source::Max.to_string()>{Source::Max.to_string()}</option>
                            <option value=Source::Netflix.to_string()>{Source::Netflix.to_string()}</option>
                            <option value=Source::Shudder.to_string()>{Source::Shudder.to_string()}</option>
                        </select>
                    </div>
                    <div class="input-group mb-3">
                        <label class="input-group-text" for="source">Source Url</label>
                        <input type="text"
                            name="source_url"
                            class="form-control"
                        />
                    </div>
                    <div class="input-group mb-3">
                        <label class="input-group-text" for="imdb">IMDB Link</label>
                        <input type="text"
                            name="imdb"
                            class="form-control"
                        />
                    </div>
                    <div class="input-group mb-3">
                        <label class="input-group-text" for="wikipedia">Wikipedia Link</label>
                        <input type="text"
                            name="wikipedia"
                            class="form-control"
                        />
                    </div>
                    <div>
                        <input type="submit"
                            class="btn btn-primary"
                            value="Add"
                        />
                    </div>
                </ActionForm>
            </div>
        </div>
    }
}

#[server(AddSeries)]
pub async fn add_series(
    name: String,
    source: Source,
    source_url: String,
    imdb: String,
    wikipedia: String,
) -> Result<(), ServerFnError> {
    use std::sync::OnceLock;

    use sqlx::{Pool, Postgres};

    use crate::db::load_statement;

    let pool = expect_context::<Pool<Postgres>>();

    static SQL: OnceLock<String> = OnceLock::new();
    let s = SQL.get_or_init(|| load_statement("insert_series.sql"));
    sqlx::query(s)
        .persistent(true)
        .bind(name)
        .bind(source.to_string())
        .bind(source_url)
        .bind(if imdb.is_empty() { None } else { Some(imdb) })
        .bind(if wikipedia.is_empty() {
            None
        } else {
            Some(wikipedia)
        })
        .execute(&pool)
        .await?;

    leptos_axum::redirect("/");

    Ok(())
}
